/*
	i2c.c --- i2c read/write functions
*/

#include <msp430.h>
#include <ti/mcu/msp430/Grace.h>
#include <stdio.h>

#include "user_define.h"

/* -------- global var. for i2c -------- */
unsigned char I2C_SLAVE_ADR;
unsigned char I2C_BUF_W[I2C_BUFSIZE];
unsigned char I2C_BUF_R[I2C_BUFSIZE];

int I2C_TxByteCnt, I2C_RxByteCnt;
unsigned char *I2C_TxDataPtr, *I2C_RxDataPtr;


/*
 * i2c send (write)
 *    return:  OK -> txlen (>0)
 *             any error -> error code (<0)
 */
int i2c_send(unsigned int slave_addr, unsigned char *txdata, int txlen)
{
	unsigned int tmout;
	int ret;
	tmout = 50000;

	UCB0I2CSA = (slave_addr >> 1);
	I2C_TxDataPtr = txdata;
	I2C_TxByteCnt = txlen;
	UCB0CTL1 |= UCTR + UCTXSTT; // I2C TX, start condition
        
	while (tmout--){
		if (I2C_TxByteCnt <= 0){
			ret = txlen;
			break;
		}
	}
	if (tmout == 0){
		ret = E_TMOUT;
		if (UCB0STAT & UCBBUSY)		ret = E_BUSY;
		if (UCB0STAT & UCNACKIFG){	ret = E_NACK;	i2c_stop();	}
	}
	return ret;		// temporary, should be timeout check
}

/*
 * i2c receive (read)
 *    return:  OK -> rxlen (>0)
 *             any error -> error code (<0)
 */
int i2c_recv(unsigned int slave_addr, unsigned char *rxdata, int rxlen)
{
	unsigned int tmout;
	int ret;
	tmout = 50000;

	UCB0I2CSA = (slave_addr >> 1);
	I2C_RxDataPtr = rxdata;
	I2C_RxByteCnt = rxlen;
	UCB0CTL1 &= ~UCTR;		// I2C receiver mode
	UCB0CTL1 |= UCTXSTT;	// I2C start condition

	if (rxlen == 1){		// in case of only 1 byte receive
		while (tmout--){
			if (!(UCB0CTL1 & UCTXSTT)){		// slave ACKed ?
				UCB0CTL1 |= UCTXSTP;		// then set STOP condotion
				break;
			}
		}
	}

	while (tmout--){
		if (I2C_RxByteCnt <= 0){
			ret = rxlen;
			break;
		}
	}
	if (tmout == 0){
		ret = E_TMOUT;
		if (UCB0STAT & UCBBUSY)		ret = E_BUSY;
		if (UCB0STAT & UCNACKIFG){	ret = E_NACK;	i2c_stop();	}
	}

	return ret;		// temporary, should be timeout check
}
/*
 * i2c stop condition
 */
void i2c_stop()
{
	UCB0CTL1 |= UCTXSTP;
}

/*
 * i2c reset
 */
void i2c_reset(void)
{
	/*
	unsigned int tmout;

	tmout = 200;
    UCB0CTL1 = UCSSEL_2 | UCSWRST;
    while (tmout--)
    	;
    UCB0CTL1 &= ~UCSWRST;
    */
    USCI_B0_graceInit();
    System_graceInit();
}



